﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;

namespace MSText2Xml
{
    class CoverageReader
    {
        private const int pageSize = 0x10000;
        public static void ReadIt(string filename)
        {

            if (null == filename)
            {
                throw new ArgumentNullException("filename");
            }
            var inFile = File.Open(filename, FileMode.Open, FileAccess.Read);
            var reader = new BinaryReader(inFile);
            var now = File.GetLastWriteTime(filename);

            var fileHeaderMark = reader.ReadInt32(0);
            if (fileHeaderMark != 0x454b494d)
                throw new ApplicationException("Header Mismatch");

            reader.BaseStream.Seek(0, SeekOrigin.Begin);
            char[] buffer = new char[0x4d28];
            reader.Read(buffer, 0, 0x4d28);

            var versionCheck1 = reader.ReadInt32(8);
            if (versionCheck1 != 0x2b)
                throw new ApplicationException("Invalid version");

            var versionCheck2 = reader.ReadInt32(0x54);
            if (versionCheck2 < 1)
                throw new ApplicationException("version check 2 fail");

            var architectureValue = reader.ReadInt32(140);
            if (architectureValue != pageSize)
            {
                throw new ApplicationException("architecture check 2 fail");
            }

            //Allocate a buffer
            byte[] fileBuffer = new byte[pageSize];

            var num20 = reader.ReadInt32(0x898);
            var blockPosition = reader.ReadInt32(0x890);

            reader.BaseStream.Seek(blockPosition, SeekOrigin.Begin);
            reader.Read(fileBuffer, 0, pageSize);
            var sanityCheck1 = reader.ReadInt32(blockPosition);
            sanityCheck1 = BitConverter.ToInt32(fileBuffer, 0);
            if (sanityCheck1 != 0x52414542)
            {
                throw new ApplicationException("buffer position isn't right");
            }
            if (BitConverter.ToInt32(fileBuffer, 16) > pageSize)
            {
                throw new ApplicationException("Page size error");
            }

            var indexBlockPosition = 0x18;
            var indexEventPosition = indexBlockPosition + 8;
            var indexSize = BitConverter.ToInt32(fileBuffer, 12);
            if (indexSize > pageSize)
            {
                indexSize = pageSize;
            }
            var indexEventEndPosition = indexEventPosition + indexSize;
            var num11 = 0;

        label_x:
            if (num11 >= indexSize || indexEventPosition >= indexEventEndPosition)
                goto label_y;
            var eventBlobPosition = indexEventPosition;
            var value = BitConverter.ToInt32(fileBuffer, eventBlobPosition);
            if ((GetEventSize(value) + indexEventPosition) > indexEventEndPosition)
            {
                throw new ApplicationException("cc.err.invalidfile");
            }


        label_y:

            return;
        }

        internal static long GetEventSize(Int32 eventValue)
{
    long num = 0;
    switch (eventValue)
    {
        case 1:
            return 0x228;

        case 2:
            return 20;

        case 3:
            num = 0x20;
            break;

        case 4:
            return 0x1c;

        case 5:
//            num = (ulong) ((lstrlenW(eventValue + 40)) * 2) + 0x2a);
            break;

        case 6:
            //if (*(((byte*) (peb + 40))) > 0x10)
            //{
            //    *((sbyte*) (peb + 40)) = 0x10;
            //}
            //num = (uint modopt(IsLong)) ((*(((byte*) (peb + 40))) * 0x10) + 0x2c);
            break;

        case 7:
            num = 0x234;
            break;

        case 8:
            num = 0x234;
            break;

        case 9:
            return 0x218;

        case 10:
            num = 0x114;
            break;

        case 11:
            return 0x2c;

        case 12:
            return num;

        case 13:
            return 0x1c;

        case 14:
            return 0x1c;

        case 15:
            return 0x23b;

        case 0x10:
            return 0x2d;

        case 0x11:
            return 60;

        case 0x12:
            //num = (uint modopt(IsLong)) ((((*(((int*) (peb + 40))) * 4) + *(((int*) (peb + 0x24)))) * 2) + 0x2c);
            break;

        case 0x13:
            num = eventValue + 4;
            break;

        default:
            return num;
    }
    if (num > 0x300)
    {
        num = 0x300;
    }
    return num;
}




    }

    public static class Extensions
    {
        public static Int32 ReadInt32(this BinaryReader source, long position)
        {
            source.BaseStream.Seek(position, SeekOrigin.Begin);
            return source.ReadInt32();
        }
    }

}