﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    public class Class
    {
        private readonly NamespaceTable namespaceTable;
        private readonly string className;
        private readonly List<Method> methods = new List<Method>();

        public Class(NamespaceTable namespaceTable, string className)
        {
            this.namespaceTable = namespaceTable;
            this.className = className;
        }

        public NamespaceTable NamespaceTable
        {
            get
            {
                return namespaceTable;
            }
        }

        public string ClassName
        {
            get
            {
                return className;
            }
        }
        public string ClassKeyName
        {
            get
            {
                return NamespaceTable.NamespaceKeyName + ClassName;
            }
        }

        public int LinesCovered
        {
            get
            {
                return methods.Sum(m => m.LinesCovered);
            }
        }

        public int LinesNotCovered
        {
            get
            {
                return methods.Sum(m => m.LinesNotCovered);
            }
        }

        public IEnumerable<Method> Methods
        {
            get
            {
                return methods;
            }
        }

        public void AddMethod(Method method)
        {
            methods.Add(method);
        }

        public XElement ToXElement()
        {
            return new XElement("Class",
                                new XElement("ClassKeyName", ClassKeyName),
                                new XElement("ClassName", ClassName),
                                new XElement("LinesCovered", LinesCovered),
                                new XElement("LinesNotCovered", LinesNotCovered),
                                new XElement("LinesPartiallyCovered", 0),
                                new XElement("BlocksCovered", LinesCovered),
                                new XElement("BlocksNotCovered", LinesNotCovered),
                                new XElement("NamespaceKeyName", NamespaceTable.NamespaceKeyName),
                                methods.OrderBy(m => m.MethodName).Select(m => m.ToXElement()));
        }
    }
}
