﻿using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    public sealed class Line
    {
        private static int lineIdCounter;
        private readonly int startLine;
        private readonly int startColumn;
        private readonly int endLine;
        private readonly int endColumn;
        private readonly bool covered;
        private readonly SourceFile sourceFile;
        private readonly int lineId;

        public Line(int startLine, int startColumn, int endLine, int endColumn, bool covered, SourceFile sourceFile)
        {
            this.startLine = startLine;
            this.startColumn = startColumn;
            this.endLine = endLine;
            this.endColumn = endColumn;
            this.covered = covered;
            this.sourceFile = sourceFile;
            lineId = lineIdCounter;
            lineIdCounter++;
        }

        public int StartLine
        {
            get
            {
                return startLine;
            }
        }

        public int StartColumn
        {
            get
            {
                return startColumn;
            }
        }

        public int EndLine
        {
            get
            {
                return endLine;
            }
        }

        public int EndColumn
        {
            get
            {
                return endColumn;
            }
        }

        public bool Covered
        {
            get
            {
                return covered;
            }
        }

        public int Coverage
        {
            get
            {
                if (covered)
                {
                    return 0;
                }
                // NCover basic doesn't support partial coverage.
                return 2;
            }
        }

        public SourceFile SourceFile
        {
            get
            {
                return sourceFile;
            }
        }

        public int LineId
        {
            get
            {
                return lineId;
            }
        }

        public XElement ToXElement()
        {
            return new XElement("Line",
                                new XElement("LnStart", StartLine),
                                new XElement("ColStart", StartColumn),
                                new XElement("LnEnd", EndLine),
                                new XElement("ColEnd", EndColumn),
                                new XElement("Coverage", Coverage),
                                new XElement("SourceFileID", SourceFile.Id),
                                new XElement("LineID", LineId));
        }
    }
}
