﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    public sealed class Method
    {
        private readonly Class @class;
        private readonly string methodName;
        private readonly string methodKeyName;
        private readonly List<Line> lines = new List<Line>();

        public Method(Class @class, string methodName)
        {
            this.@class = @class;
            this.methodName = methodName;
            methodKeyName = string.Concat(@class.NamespaceTable.Module.ModuleName, "!", @class.NamespaceTable.Module.NextUniqueMethodKey().ToString("X").PadLeft(4, '0'));
        }

        public Class Class
        {
            get
            {
                return @class;
            }
        }

        public string MethodName
        {
            get
            {
                return methodName;
            }
        }

        public string MethodNameKey
        {
            get
            {
                return methodKeyName;
            }
        }

        public int LinesCovered
        {
            get
            {
                return lines.Where(l => l.Covered).Count();
            }
        }
        public int LinesNotCovered
        {
            get
            {
                return lines.Where(l => !l.Covered).Count();
            }
        }

        public void AddLine(Line line)
        {
            lines.Add(line);
        }

        public XElement ToXElement()
        {
            return new XElement("Method",
                                new XElement("MethodKeyName", MethodNameKey),
                                new XElement("MethodName", MethodName),
                                new XElement("MethodFullName", MethodName),
                                new XElement("LinesCovered", LinesCovered),
                                new XElement("LinesPartiallyCovered", 0),
                                new XElement("LinesNotCovered", LinesNotCovered),
                                new XElement("BlocksCovered", LinesCovered),
                                new XElement("BlocksNotCovered", LinesNotCovered),
                                lines.OrderBy(l => l.StartLine).Select(l => l.ToXElement()));
        }
    }
}
