﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    public sealed class Module
    {
        private static int uniqueKeyCounter;
        private readonly CoverageDSPriv msCover;
        private readonly string moduleName;
        private readonly Dictionary<string, NamespaceTable> namespaceTables = new Dictionary<string, NamespaceTable>();
        private readonly Dictionary<string, Class> classes = new Dictionary<string, Class>();

        public Module(CoverageDSPriv msCover, string moduleName)
        {
            this.msCover = msCover;
            this.moduleName = moduleName;
        }

        public CoverageDSPriv MSCover
        {
            get
            {
                return msCover;
            }
        }

        public string ModuleName
        {
            get
            {
                return moduleName;
            }
        }

        public int LinesCovered
        {
            get
            {
                return namespaceTables.Values.Sum(n => n.LinesCovered);
            }
        }

        public int LinesNotCovered
        {
            get
            {
                return namespaceTables.Values.Sum(n => n.LinesNotCovered);
            }
        }

        public IEnumerable<NamespaceTable> NamespaceTables
        {
            get
            {
                return namespaceTables.Values;
            }
        }

        public XElement ToXElement()
        {
            return new XElement("Module",
                                new XElement("ModuleName", ModuleName),
                                new XElement("ImageSize", 49152),
                                new XElement("ImageLinkTime", 0),
                                new XElement("LinesCovered", LinesCovered),
                                new XElement("LinesPartiallyCovered", 0),
                                new XElement("LinesNotCovered", LinesNotCovered),
                                new XElement("BlocksCovered", LinesCovered),
                                new XElement("BlocksNotCovered", LinesNotCovered),
                                namespaceTables.Values.OrderBy(n => n.NamespaceName).Select(n => n.ToXElement()));
        }

        public Class GetClass(string fullName)
        {
            Class @class;
            if (classes.TryGetValue(fullName, out @class))
            {
                return @class;
            }

            string namespaceName = fullName.Substring(0, fullName.LastIndexOf('.'));

            NamespaceTable namespaceTable;
            if (!namespaceTables.TryGetValue(namespaceName, out namespaceTable))
            {
                namespaceTable = new NamespaceTable(this, namespaceName);
                namespaceTables.Add(namespaceName, namespaceTable);
            }

            string className = fullName.Substring(fullName.LastIndexOf('.') + 1);

            @class = new Class(namespaceTable, className);
            classes.Add(fullName, @class);
            namespaceTable.AddClass(@class);

            return @class;
        }

        public int NextUniqueMethodKey()
        {
            uniqueKeyCounter++;
            return uniqueKeyCounter;
        }
    }
}
