﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    public sealed class NamespaceTable
    {
        private readonly Module module;
        private readonly string namespaceName;
        private readonly List<Class> classes = new List<Class>();

        public NamespaceTable(Module module, string namespaceName)
        {
            this.module = module;
            this.namespaceName = namespaceName;
        }

        public Module Module
        {
            get
            {
                return module;
            }
        }

        public string NamespaceName
        {
            get
            {
                return namespaceName;
            }
        }

        public string NamespaceKeyName
        {
            get
            {
                return ModuleName + NamespaceName;
            }
        }

        private string ModuleName
        {
            get
            {
                return Module.ModuleName;
            }
        }

        public int LinesCovered
        {
            get
            {
                return classes.Sum(c => c.LinesCovered);
            }
        }

        public int LinesNotCovered
        {
            get
            {
                return classes.Sum(c => c.LinesNotCovered);
            }
        }

        public IEnumerable<Class> Classes
        {
            get
            {
                return classes;
            }
        }

        public void AddClass(Class @class)
        {
            classes.Add(@class);
        }

        public XElement ToXElement()
        {
            return new XElement("NamespaceTable",
                                new XElement("BlocksCovered", LinesCovered),
                                new XElement("BlocksNotCovered", LinesNotCovered),
                                new XElement("LinesCovered", LinesCovered),
                                new XElement("LinesNotCovered", LinesNotCovered),
                                new XElement("LinesPartiallyCovered", 0),
                                new XElement("ModuleName", ModuleName),
                                new XElement("NamespaceKeyName", NamespaceKeyName),
                                new XElement("NamespaceName", NamespaceName),
                                classes.OrderBy(c => c.ClassName).Select(c => c.ToXElement()));
        }
    }
}
