﻿using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Xml.Linq;

namespace NUnit4TeamBuild.NCoverToMSCover
{
    internal static class Program
    {
        private static void Main()
        {
            XDocument ncoverXml = XDocument.Load(@"C:\PD\nunit4teambuild\NCoverToMSCover\NCoverToMSCover\NCover.xml");

            var coverage = CreateCoverage(ncoverXml);

            XDocument output = new XDocument(coverage.ToXElement());
            AttachSchema(output);
            output.Save(@"J:\Development\NCoverToMSCover\NCoverToMSCover\NCoverToMSCover.xml");
        }

        private static CoverageDSPriv CreateCoverage(XDocument ncoverXml)
        {
            CoverageDSPriv coverage = new CoverageDSPriv();

            AddModules(coverage, ncoverXml.Root.Elements("module"));

            return coverage;
        }
        private static void AddModules(CoverageDSPriv coverage, IEnumerable<XElement> moduleElements)
        {
            foreach (XElement moduleElement in moduleElements)
            {
                string ncoverName = moduleElement.Attribute("name").Value;
                Module module = new Module(coverage, ncoverName.Substring(ncoverName.LastIndexOf('\\') + 1));
                coverage.AddModule(module);

                AddMethods(coverage, module, moduleElement.Elements("method"));
            }
        }
        private static void AddMethods(CoverageDSPriv coverage, Module module, IEnumerable<XElement> methodElements)
        {
            foreach (XElement methodElement in methodElements)
            {
                Class @class = module.GetClass(methodElement.Attribute("class").Value);

                Method method = new Method(@class, methodElement.Attribute("name").Value + "()");
                @class.AddMethod(method);

                AddLines(coverage, method, methodElement.Elements("seqpnt"));
            }
        }
        private static void AddLines(CoverageDSPriv coverage, Method method, IEnumerable<XElement> seqpntElements)
        {
            foreach (XElement seqpntElement in seqpntElements)
            {
                SourceFile sourceFile = coverage.GetSourceFile(seqpntElement.Attribute("document").Value);
                
                bool isCovered = seqpntElement.Attribute("visitcount").Value != "0";
                int startLine = int.Parse(seqpntElement.Attribute("line").Value);
                int startColumn = int.Parse(seqpntElement.Attribute("column").Value);
                int endLine = int.Parse(seqpntElement.Attribute("endline").Value);
                int endColumn = int.Parse(seqpntElement.Attribute("endcolumn").Value);

                Line line = new Line(startLine, startColumn, endLine, endColumn, isCovered, sourceFile);

                method.AddLine(line);
            }
        }
        private static void AttachSchema(XDocument xml)
        {
            using (var stream = typeof(Program).Assembly.GetManifestResourceStream("NUnit4TeamBuild.NCoverToMSCover.CoverageSchema.xsd"))
            {
                using (var reader = XmlReader.Create(stream))
                {
                    var schema = XElement.Load(reader);
                    xml.Root.AddFirst(schema);
                }
            }
        }
    }
}