﻿using System;
using CommandLineParser.Arguments;
using CommandLineParser.Exceptions;

namespace KWatkins.NUnitTfs
{
    public sealed class Options
    {
        private Options()
        {
            Platform = "Any CPU";
            Flavour = "Release";
            OutputFile = "NUnitOutput.trx";
            TfsVersion = 2008;
        }

        [ValueArgument(typeof(string), 'n', "nunit", Description = "Path to NUnit output file", Optional = false)]
        public string NUnitOutput { get; set; }

        [ValueArgument(typeof(string), 'u', "serveruri", Description = "Team Foundation Server or Collection uri, e.g. http://tfs:8080/tfs/MyCollection")]
        public string ServerUri { get; set; }

        [ValueArgument(typeof(string), 't', "teamproject", Description = "Team Foundation project name, e.g. MyProject", Optional = false)]
        public string TeamProject { get; set; }

        [ValueArgument(typeof(string), 'p', "platform", Description = "Platform. Defaults to \"Any CPU\"")]
        public string Platform { get; set; }

        [ValueArgument(typeof(string), 'f', "flavour", Description = "Flavour. Defaults to \"Release\"")]
        public string Flavour { get; set; }

        [ValueArgument(typeof(string), 'b', "build", Description = "Name of the build to publish to, e.g. Release_20090608.5", Optional = false)]
        public string Build { get; set; }

        [ValueArgument(typeof(string), 'o', "outputFile", Description = "Name of the test run output file. Defaults to \"NUnitOutput.trx\"")]
        public string OutputFile { get; set; }

        [EnumeratedValueArgument(typeof(int), 'v', "version", Description = "Version of TFS, either 2008 or 2010. Defaults to 2008.", AllowedValues = "2008;2010")]
        public int TfsVersion { get; set; }

        [ValueArgument(typeof(string), 'x', "xslt", Description = "Path of the NUnitToMSTest.xslt file. If not supplied then the XSLTs built into NUnitTFS.exe will be used.")]
        public string NUnitToMSTestXSLT { get; set; }

        [SwitchArgument('c', "convertonly", false, Description = "Convert the NUnit output file to an MSTest TRX file without publishing.")]
        public bool ConvertOnly { get; set; }

        internal static Options ParseOptions(string[] args)
        {
            CommandLineParser.CommandLineParser parser = new CommandLineParser.CommandLineParser();
            Options options = new Options();
            parser.ExtractArgumentAttributes(options);

            try
            {
                parser.ParseCommandLine(args);
                return options;
            }
            catch (CommandLineException e)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.Error.WriteLine(e.Message);
                Console.Error.WriteLine();
                Console.ForegroundColor = ConsoleColor.White;
                parser.ShowUsage();
                return null;
            }
        }
    }
}
