﻿using System;
using System.IO;
using System.Linq;
using System.Xml.Linq;

namespace KWatkins.NUnitTfs.Tfs2008
{
    internal sealed class Publisher2008 : Publisher
    {
        private const BuildServiceV2.BuildStatus BuildStatusAll =
            BuildServiceV2.BuildStatus.InProgress |
            BuildServiceV2.BuildStatus.Succeeded |
            BuildServiceV2.BuildStatus.PartiallySucceeded |
            BuildServiceV2.BuildStatus.Failed |
            BuildServiceV2.BuildStatus.Stopped |
            BuildServiceV2.BuildStatus.NotStarted;

        internal Publisher2008(Options options)
            : base(options)
        {
        }

        protected override void PublishToTfs(XDocument trx)
        {
            Console.WriteLine("QueryBuilds");
            var buildQueryResult = GetBuildDetails(Options.TeamProject, Options.Build);

            string buildId = GetBuildId(buildQueryResult.Builds[0]);

            Guid runId = GetRunId(trx);

            Console.WriteLine("GetVersion");
            string publishServiceVersion = GetPublishServiceVersion();

            Console.WriteLine("PrepareToPublish");
            string resultsDirectory = PrepareToPublish(buildId, runId, false);

            var trxPath = Path.Combine(resultsDirectory, Options.OutputFile);
            Console.WriteLine("Writing TRX file to " + trxPath);
            trx.Save(trxPath);

            Console.WriteLine("PublishRun");
            PublishRun(buildId, runId);
        }


        private void PublishRun(string buildId, Guid runId)
        {
            using (var client = new PublishTestResultsBuildService2V1.Service2SoapClient())
            {
                SetEndpointAddress(client.Endpoint, "Build/v1.0/PublishTestResultsBuildService2.asmx");
                var error = client.PublishRun(buildId, Options.OutputFile, runId);
                if (error != null)
                {
					throw new InvalidOperationException(string.Concat("PublishRun failed: ", error.Name));
                }
            }
        }

        private string PrepareToPublish(string buildId, Guid runId, bool overwrite)
        {
            using (var client = new PublishTestResultsBuildService2V1.Service2SoapClient())
            {
                SetEndpointAddress(client.Endpoint, "Build/v1.0/PublishTestResultsBuildService2.asmx");
                string resultsDirectory;
                var error = client.PrepareToPublish(buildId, runId, overwrite, out resultsDirectory);
                if (error != null)
                {
                    throw new InvalidOperationException(string.Concat("PrepareToPublish failed: ", error.Name));
                }
                return resultsDirectory;
            }
        }

        private string GetPublishServiceVersion()
        {
            using (var client = new PublishTestResultsBuildService2V1.Service2SoapClient())
            {
                SetEndpointAddress(client.Endpoint, "Build/v1.0/PublishTestResultsBuildService2.asmx");
                return client.GetVersion();
            }
        }

        private BuildServiceV2.BuildQueryResult GetBuildDetails(string teamProject, string buildNumber)
        {
            using (var buildClient = new BuildServiceV2.BuildServiceSoapClient())
            {
                SetEndpointAddress(buildClient.Endpoint, "Build/v2.0/BuildService.asmx");
                var buildDetailSpec = new BuildServiceV2.BuildDetailSpec
                {
                    DefinitionPath = string.Format(@"\{0}\*", teamProject),
                    BuildNumber = buildNumber,
                    Status = BuildStatusAll,
                    DefinitionSpec = new BuildServiceV2.BuildDefinitionSpec
                    {
                        FullPath = string.Format(@"\{0}\*", teamProject)
                    }
                };

                var specs = new[] { buildDetailSpec };

                var queryBuildsResult = buildClient.QueryBuilds(specs);

                return queryBuildsResult[0];
            }
        }

        private static string GetBuildId(BuildServiceV2.BuildDetail buildDetail)
        {
            var buildUri = new Uri(buildDetail.Uri);
            return buildUri.Segments.Last();
        }
    }
}
